//************************************************************************
// Example3GCD.java
//
// Presents and tests a few algorithms for finding the greatest common
// divisor of two integers.
//************************************************************************

class Example3GCD
{
   //=====================================================================
   //  Examines several methods for finding greatest common divisor.
   //=====================================================================
   public static void main (String[] args)
   {
      String[] AppliedMethod = {"very simple", "simple", "basic Euclidean",
                                "Euclidean", "inexistent"};
      int m = (int) (100 * Math.random());
      int n = (int) (100 * Math.random());
      if (m>0 || n >0)
      {
         System.out.println ("The first number is " + m +
                             "\nThe second number is " + n);
         for (int j=0; j <= 4; j++)
            System.out.println ("The " + AppliedMethod[j] +
                                " method gives: gcd = " +
                                NewGCD.find(AppliedMethod[j], m, n));
      }
      else 
         System.out.println ("Undefined if both are 0");

      // Compare runtimes of methods.
      long time;
      final int NUMBER_OF_TRIALS = 50000;
      System.out.println ("\nRuntimes for " + NUMBER_OF_TRIALS +
                          " pairs of integers are:");

      for (int j=0; j <= 3; j++)
      {
         time = System.currentTimeMillis();
         for (int i=1; i <= NUMBER_OF_TRIALS; i++)
         { 
            m = (int) (Short.MAX_VALUE * Math.random());
            n = (int) (Short.MAX_VALUE * Math.random());
            NewGCD.find (AppliedMethod[j], m, n);
         }
         System.out.println ("By the " + AppliedMethod[j] + " method: " +
                             (System.currentTimeMillis() - time));
      }
   
   }
}// class Example3GCD


class NewGCD
{
   //=====================================================================
   //  Contains a method with 4 variants for finding the g.c.d. of two
   //  integers. Both integers are non-negative, at least one positive.
   //=====================================================================
   public static int find (String whichMethod, int m, int n)
   {
      int gcd = 0;
      char firstLetter = whichMethod.charAt(0);
      switch (firstLetter)
      {
         case 'v':
         // Tries all integers from 1 up to m, and takes largest divisor
         int divisor = 1;
         for (int i=1; i <= m; i++)
            if ((m % i == 0) && (n % i == 0))
               divisor = i;
         gcd = divisor;
         break;

         case 's':
         // Tries all integers from min(m,n) down to 1
         int candidate = Math.min(m, n);
         while ((candidate > 1) &&
                ((m % candidate != 0) || (n % candidate != 0)))
            candidate--;
         gcd = candidate;
         break;

         case 'b':
         // Replaces the larger integer by the difference between the two
         // until the smaller becomes 0
         while ((m > 0) && (n>0))
            if (m > n)
               m -= n;
            else
               n -= m;
         gcd = m + n;
         break;

         case 'E':
         // Replaces the larger integer by its remainder modulo the
         // smaller until the smaller becomes 0
         while ((m > 0) && (n>0))
            if (m > n)
               m %= n;
            else
               n %= m;
         gcd = m + n;
         break;

         default:
         System.out.println ("No such method, gcd will be given as 0.");
         break;
      }
 
      return gcd;
   }
}// class NewGCD
